@@ -41,8 +41,7 @@ static int Multiply(void), Divide(void), Mod(void), Add(void),
     UnMinus(void), LogNot(void),
     Compare(int);
 
-static int MakeValue (char const *s, Value *v, Var *locals);
-static int ParseLiteralDate (char const **s, int *jul, int *tim);
+static int MakeValue (char const *s, Value *v, Var *locals, ParsePtr p);
 
 /* Binary operators - all left-associative */
 
@@ -73,9 +72,7 @@ Operator UnOp[] = {
 };
 #define NUM_UN_OPS (sizeof(UnOp) / sizeof(Operator))
 
-/* Functions have the same definitions as operators, except the prec field
-   is used to indicate how many arguments are needed. */
-extern Operator Func[];
+extern BuiltinFunc Func[];
 
 Operator OpStack[OP_STACK_SIZE];
 Value    ValStack[VAL_STACK_SIZE];
@@ -135,7 +132,7 @@ static void CleanStack(void)
 static char PeekChar(char const **s)
 {
     char const *t = *s;
-    while (*t && isspace(*t)) t++;
+    while (*t && isempty(*t)) t++;
     return *t;
 }
 
@@ -153,7 +150,7 @@ static int ParseExprToken(DynamicBuffer *buf, char const **in)
 
     DBufFree(buf);
 /* Skip white space */
-    while (**in && isspace(**in)) (*in)++;
+    while (**in && isempty(**in)) (*in)++;
 
     if (!**in) return OK;
 
@@ -286,7 +283,7 @@ static int ParseExprToken(DynamicBuffer *buf, char const **in)
 	(*in)++;
     }
     /* Chew up any remaining white space */
-    while (**in && isspace(**in)) (*in)++;
+    while (**in && isempty(**in)) (*in)++;
 
     /* Peek ahead - is it '('?  Then we have a function call */
     if (**in == '(') {
@@ -306,14 +303,14 @@ static int ParseExprToken(DynamicBuffer *buf, char const **in)
 /*  Put the result into value pointed to by v.                 */
 /*                                                             */
 /***************************************************************/
-int EvalExpr(char const **e, Value *v)
+int EvalExpr(char const **e, Value *v, ParsePtr p)
 {
     int r;
 
     OpStackPtr = 0;
     ValStackPtr = 0;
 
-    r = Evaluate(e, NULL);
+    r = Evaluate(e, NULL, p);
 
     /* Put last character parsed back onto input stream */
     if (DBufLen(&ExprBuf)) (*e)--;
@@ -329,11 +326,12 @@ int EvalExpr(char const **e, Value *v)
 }
 
 /* Evaluate - do the actual work of evaluation. */
-int Evaluate(char const **s, Var *locals)
+int Evaluate(char const **s, Var *locals, ParsePtr p)
 {
     int OpBase, ValBase;
     int r;
-    Operator *f;
+    Operator *o;
+    BuiltinFunc *f;
     int args; /* Number of function arguments */
     Operator op, op2;
     Value va;
@@ -353,7 +351,7 @@ int Evaluate(char const **s, Var *locals)
 
 	if (*DBufValue(&ExprBuf) == '(') { /* Parenthesized expression */
 	    DBufFree(&ExprBuf);
-	    r = Evaluate(s, locals);  /* Leaves the last parsed token in ExprBuf */
+	    r = Evaluate(s, locals, p);  /* Leaves the last parsed token in ExprBuf */
 	    if (r) return r;
 	    r = OK;
 	    if (*DBufValue(&ExprBuf) != ')') {
@@ -376,9 +374,11 @@ int Evaluate(char const **s, Var *locals)
 	    }
 	    args = 0;
 	    if (PeekChar(s) == ')') { /* Function has no arguments */
-		if (f) r = CallFunc(f, 0);
-		else {
-		    r = CallUserFunc(ufname, 0);
+		if (f) {
+		    if (!f->is_constant && (p != NULL)) p->nonconst_expr = 1;
+		    r = CallFunc(f, 0);
+		} else {
+		    r = CallUserFunc(ufname, 0, p);
 		    free((char *) ufname);
 		}
 		if (r) return r;
@@ -387,7 +387,7 @@ int Evaluate(char const **s, Var *locals)
 	    } else { /* Function has some arguments */
 		while(1) {
 		    args++;
-		    r = Evaluate(s, locals);
+		    r = Evaluate(s, locals, p);
 		    if (r) {
 			if (!f) free((char *) ufname);
 			return r;
@@ -401,19 +401,21 @@ int Evaluate(char const **s, Var *locals)
 			return E_EXPECT_COMMA;
 		    }
 		}
-		if (f) r = CallFunc(f, args);
-		else {
-		    r = CallUserFunc(ufname, args);
+		if (f) {
+		    if (!f->is_constant && (p != NULL)) p->nonconst_expr = 1;
+		    r = CallFunc(f, args);
+		} else {
+		    r = CallUserFunc(ufname, args, p);
 		    free((char *) ufname);
 		}
 		DBufFree(&ExprBuf);
 		if (r) return r;
 	    }
 	} else { /* Unary operator */
-	    f = FindFunc(DBufValue(&ExprBuf), UnOp, NUM_UN_OPS);
-	    if (f) {
+	    o = FindOperator(DBufValue(&ExprBuf), UnOp, NUM_UN_OPS);
+	    if (o) {
 		DBufFree(&ExprBuf);
-		PushOpStack(*f);
+		PushOpStack(*o);
 		continue;  /* Still looking for an atomic vlue */
 	    } else if (!ISID(*DBufValue(&ExprBuf)) &&
 		       *DBufValue(&ExprBuf) != '$' &&
@@ -424,7 +426,7 @@ int Evaluate(char const **s, Var *locals)
 		DBufFree(&ExprBuf);
 		return E_ILLEGAL_CHAR;
 	    } else { /* Must be a literal value */
-		r = MakeValue(DBufValue(&ExprBuf), &va, locals);
+		r = MakeValue(DBufValue(&ExprBuf), &va, locals, p);
 		DBufFree(&ExprBuf);
 		if (r) return r;
 		PushValStack(va);
@@ -459,13 +461,13 @@ int Evaluate(char const **s, Var *locals)
 	    return OK;
 	}
 	/* Must be a binary operator */
-	f = FindFunc(DBufValue(&ExprBuf), BinOp, NUM_BIN_OPS);
+	o = FindOperator(DBufValue(&ExprBuf), BinOp, NUM_BIN_OPS);
 	DBufFree(&ExprBuf);
-	if (!f) return E_EXPECTING_BINOP;
+	if (!o) return E_EXPECTING_BINOP;
 
 	/* While operators of higher or equal precedence are on the stack,
 	   pop them off and evaluate */
-	while (OpStackPtr > OpBase && OpStack[OpStackPtr-1].prec >= f->prec) {
+	while (OpStackPtr > OpBase && OpStack[OpStackPtr-1].prec >= o->prec) {
 	    PopOpStack(op2);
 	    if (r) return r;
 	    if (DebugFlag & DB_PRTEXPR)
@@ -477,7 +479,7 @@ int Evaluate(char const **s, Var *locals)
 		return r;
 	    }
 	}
-	PushOpStack(*f);
+	PushOpStack(*o);
     }
 }
 
@@ -488,7 +490,7 @@ int Evaluate(char const **s, Var *locals)
 /*  a date or the value of a symbol.                           */
 /*                                                             */
 /***************************************************************/
-static int MakeValue(char const *s, Value *v, Var *locals)
+static int MakeValue(char const *s, Value *v, Var *locals, ParsePtr p)
 {
     int len;
     int h, m, r;
@@ -543,6 +545,7 @@ static int MakeValue(char const *s, Value *v, Var *locals)
 	v->v.val = len;
 	return OK;
     } else if (*s == '$') { /* A system variable */
+	if (p) p->nonconst_expr = 1;
 	if (DebugFlag & DB_PRTEXPR)
 	    fprintf(ErrFp, "%s => ", s);
 	r = GetSysVar(s+1, v);
@@ -553,10 +556,11 @@ static int MakeValue(char const *s, Value *v, Var *locals)
 	    Putc('\n', ErrFp);
 	}
 	return r;
-    } else /* Must be a symbol */
+    } else { /* Must be a symbol */
 	if (DebugFlag & DB_PRTEXPR)
 	    fprintf(ErrFp, "%s => ", s);
-    r = GetVarValue(s, v, locals);
+    }
+    r = GetVarValue(s, v, locals, p);
     if (! (DebugFlag & DB_PRTEXPR)) return r;
     if (r == OK) {
 	PrintValue(v, ErrFp);
@@ -737,6 +741,8 @@ static int Add(void)
     Value v1, v2, v3;
     int r;
 
+    size_t l1, l2;
+
     PopValStack(v2);
     if ( (r = FnPopValStack(&v1)) ) {
 	DestroyValue(v2);
@@ -791,7 +797,13 @@ static int Add(void)
 	    return r;
 	}
 	v3.type = STR_TYPE;
-	v3.v.str = malloc(strlen(v1.v.str) + strlen(v2.v.str) + 1);
+	l1 = strlen(v1.v.str);
+	l2 = strlen(v2.v.str);
+	if (MaxStringLen && (l1 + l2 > MaxStringLen)) {
+	    DestroyValue(v1); DestroyValue(v2);
+	    return E_STRING_TOO_LONG;
+	}
+	v3.v.str = malloc(l1 + l2 + 1);
 	if (!v3.v.str) {
 	    DestroyValue(v1); DestroyValue(v2);
 	    return E_NO_MEM;
@@ -1118,7 +1130,28 @@ static int LogNot(void)
 /*  Find a function.                                           */
 /*                                                             */
 /***************************************************************/
-Operator *FindFunc(char const *name, Operator where[], int num)
+Operator *FindOperator(char const *name, Operator where[], int num)
+{
+    int top=num-1, bot=0;
+    int mid, r;
+    while (top >= bot) {
+	mid = (top + bot) / 2;
+	r = StrCmpi(name, where[mid].name);
+	if (!r) return &where[mid];
+	else if (r > 0) bot = mid+1;
+	else top = mid-1;
+    }
+    return NULL;
+}
+
+/***************************************************************/
+/*                                                             */
+/*  FindFunc                                                   */
+/*                                                             */
+/*  Find a function.                                           */
+/*                                                             */
+/***************************************************************/
+BuiltinFunc *FindFunc(char const *name, BuiltinFunc where[], int num)
 {
     int top=num-1, bot=0;
     int mid, r;
@@ -1194,7 +1227,7 @@ int CopyValue(Value *dest, const Value *src)
 /*  and tim; update s.                                         */
 /*                                                             */
 /***************************************************************/
-static int ParseLiteralDate(char const **s, int *jul, int *tim)
+int ParseLiteralDate(char const **s, int *jul, int *tim)
 {
     int y, m, d;
     int hour, min;
