@@ -1854,10 +1854,11 @@ static int FTimeStuff(int wantmins, func_info *info)
 
 static int SunStuff(int rise, double cosz, int jul)
 {
-    int year, mon, day;
-    int jan0;
     int mins, hours;
     int dusk_or_dawn;
+    int year, mon, day;
+    int jan0;
+    double jan0d;
 
     double M, L, tanA, sinDelta, cosDelta, a, a_hr, cosH, t, H, T;
     double latitude, longdeg, UT, local;
@@ -1880,25 +1881,34 @@ static int SunStuff(int rise, double cosz, int jul)
 
     FromJulian(jul, &year, &mon, &day);
     jan0 = jul - Julian(year, 0, 1);
+    jan0d = (double) jan0;
 
     dusk_or_dawn = rise;
     if (rise > 1)
 	rise -= 2;
-/* Following formula on page B6 exactly... */
-    t = (double) jan0;
-    if (rise) t += (6.0 + longdeg/15.0) / 24.0;
-    else      t += (18.0 + longdeg/15.0) / 24.0;
 
-/* Mean anomaly of sun for 1978 ... how accurate for other years??? */
-    M = 0.985600 * t - 3.251;  /* In degrees */
+/* Following formula on page B6 exactly... */
+    t = (double) jul;
+    if (rise) {
+	t += (6.0 + longdeg/15.0) / 24.0;
+	jan0d += (6.0 + longdeg/15.0) / 24.0;
+    } else {
+	t += (18.0 + longdeg/15.0) / 24.0;
+	jan0d += (18.0 + longdeg/15.0) / 24.0;
+    }
+
+/* Mean anomaly of sun starting from 1 Jan 1990 */
+/* NOTE: This assumes that BASE = 1990!!! */
+#if BASE != 1990
+#error Sun calculations assume a BASE of 1990!
+#endif
+    M = (0.9856002585 * t) + 357.828757;  /* In degrees */
+
+    /* Make sure M is in the range [0, 360) */
+    M -= (floor(M/360.0) * 360.0);
 
 /* Sun's true longitude */
     L = M + 1.916*sin(DEGRAD*M) + 0.02*sin(2*DEGRAD*M) + 282.565;
-    if (dusk_or_dawn == 2) {/* dusk */
-	L += 6;
-    } else if (dusk_or_dawn == 3) {/* dawn */
-	L -= 14;
-    }
     if (L > 360.0) L -= 360.0;
 
 /* Tan of sun's right ascension */
@@ -1941,7 +1951,7 @@ static int SunStuff(int rise, double cosz, int jul)
     H = RADDEG * acos(cosH);
     if (rise) H = 360.0 - H;
 
-    T = H / 15.0 + a_hr - 0.065710*t - 6.620;
+    T = H / 15.0 + a_hr - 0.065710 * jan0d - 6.620;
     if (T >= 24.0) T -= 24.0;
     else if (T < 0.0) T+= 24.0;
     
@@ -1952,6 +1962,9 @@ static int SunStuff(int rise, double cosz, int jul)
     if (local < 0.0) local += 24.0;
     else if (local >= 24.0) local -= 24.0;
 
+    /* Round off local time to nearest minute */
+    local = floor(local * 60.0 + 0.5) / 60.0;
+
     hours = (int) local;
     mins = (int) ((local - hours) * 60.0);
     
@@ -1975,9 +1988,13 @@ static int SunStuff(int rise, double cosz, int jul)
 static int FSun(int rise, func_info *info)
 {
     int jul = JulianToday;
-    static double cosz = -0.014543897;  /* for sunrise and sunset */
+    double cosz = -0.014543897;  /* for sunrise and sunset */
     int r;
 
+    /* Civil twilight: cos(96 degrees) */
+    if (rise == 2 || rise == 3) {
+	cosz = -0.104528463268;
+    }
     if (Nargs >= 1) {
 	if (!HASDATE(ARG(0))) return E_BAD_TYPE;
 	jul = DATEPART(ARG(0));
