@@ -8,7 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*  Copyright (C) 1992-1998 by David F. Skoll                  */
-/*  Copyright (C) 1999-2010 by Roaring Penguin Software Inc.   */
+/*  Copyright (C) 1999-2011 by Roaring Penguin Software Inc.   */
 /*                                                             */
 /***************************************************************/
 
@@ -137,7 +137,7 @@ void InitRemind(int argc, char const *argv[])
     char const *s;
     int weeks;
 
-    int jul, tim;
+    int jul;
 
 #if defined(__APPLE__)
     rkrphgvba(0);
@@ -146,7 +146,6 @@ void InitRemind(int argc, char const *argv[])
 #endif
 
     jul = NO_DATE;
-    tim = NO_TIME;
 
     /* Initialize global dynamic buffers */
     DBufInit(&Banner);
@@ -318,7 +317,7 @@ void InitRemind(int argc, char const *argv[])
 
 	    case 'a':
 	    case 'A':
-		DontIssueAts = 1;
+		DontIssueAts++;
 		break;
 
 	    case 'q':
@@ -570,12 +569,12 @@ void InitRemind(int argc, char const *argv[])
 	    }
 	    if (d == NO_DAY) d=1;
 	    if (d > DaysInMonth(m, y)) {
-		fprintf(ErrFp, BadDate);
+		fprintf(ErrFp, "%s", BadDate);
 		Usage();
 	    }
 	    JulianToday = Julian(y, m, d);
 	    if (JulianToday == -1) {
-		fprintf(ErrFp, BadDate);
+		fprintf(ErrFp, "%s", BadDate);
 		Usage();
 	    }
 	    CurYear = y;
@@ -602,7 +601,7 @@ void InitRemind(int argc, char const *argv[])
 void Usage(void)
 {
     fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-1998 David F. Skoll\n", VERSION, L_LANGNAME);
-    fprintf(ErrFp, "Copyright 1999-2010 Roaring Penguin Software Inc.\n");
+    fprintf(ErrFp, "Copyright 1999-2011 Roaring Penguin Software Inc.\n");
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETA VERSION <<<<\n");
 #endif
@@ -678,7 +677,7 @@ static void ChgUser(char const *user)
 
     home = malloc(strlen(pwent->pw_dir) + 6);
     if (!home) {
-	fprintf(ErrFp, ErrMsg[M_NOMEM_ENV]);
+	fprintf(ErrFp, "%s", ErrMsg[M_NOMEM_ENV]);
 	exit(1);
     }
     sprintf(home, "HOME=%s", pwent->pw_dir);
@@ -686,7 +685,7 @@ static void ChgUser(char const *user)
 
     shell = malloc(strlen(pwent->pw_shell) + 7);
     if (!shell) {
-	fprintf(ErrFp, ErrMsg[M_NOMEM_ENV]);
+	fprintf(ErrFp, "%s", ErrMsg[M_NOMEM_ENV]);
 	exit(1);
     }
     sprintf(shell, "SHELL=%s", pwent->pw_shell);
@@ -695,14 +694,14 @@ static void ChgUser(char const *user)
     if (pwent->pw_uid) {
 	username = malloc(strlen(pwent->pw_name) + 6);
 	if (!username) {
-	    fprintf(ErrFp, ErrMsg[M_NOMEM_ENV]);
+	    fprintf(ErrFp, "%s", ErrMsg[M_NOMEM_ENV]);
 	    exit(1);
 	}
 	sprintf(username, "USER=%s", pwent->pw_name);
 	putenv(username);
 	logname= malloc(strlen(pwent->pw_name) + 9);
 	if (!logname) {
-	    fprintf(ErrFp, ErrMsg[M_NOMEM_ENV]);
+	    fprintf(ErrFp, "%s", ErrMsg[M_NOMEM_ENV]);
 	    exit(1);
 	}
 	sprintf(logname, "LOGNAME=%s", pwent->pw_name);
